module.exports = function(grunt) {
  var cheerio = require('cheerio')
  var fs = require('fs')
  var cfg = JSON.parse(fs.readFileSync('./package.json'));
  var name = cfg.name;
  var dest = {
    demo: "../../../codePub/trunk/demo",
    project: "../../../codePub/trunk/project"
  };

  /**
   * 找到公共组件的css和js文件,并放入cssArr 和 jsArr数组
   * @type {Array}
   */

  var cssArr = [];
  var jsArr = [];
  (function getHtmls() {
    var files = fs.readdirSync('./');
    var filesArr = [];
    htmlReg = /^[^_].*\.html(?:\?.*|$)/
    files.forEach(function(v, k) {
      if (htmlReg.test(v)) {
        filesArr.push(v);
      };
    })
    return filesArr;
  })()
    .forEach(function(v, k) {
      var $ = cheerio.load(fs.readFileSync(v).toString());
      var dom = $.html();
      var cssReg = /.*components.*\.css$/;
      var jsReg = /.*components.*\.js$/;
      $('link')
        .filter(function() {
          if (this.attr('href') != undefined) {
            return !this.attr('href').match(/^http|https/) && (this.attr('rel') == 'stylesheet') && this.attr('href') ? /.css$/.test(this.attr('href')) : false;
          };
        })
        .each(function(v, k) {
          if (cssReg.test(this.attr('href'))) {
            cssArr.push(this.attr('href'))
          };
        })
      $('script')
        .filter(function() {
          if (this.attr('src') != undefined) {
            return !this.attr('src').match(/^http|https/) && (this.attr('type') == 'text/javascript' || !this.attr('type')) && this.attr('src') ? /.js$/.test(this.attr('src')) : false;
          };
        })
        .each(function(v, k) {
          if (jsReg.test(this.attr('src'))) {
            jsArr.push(this.attr('src'))
          };
        })

    })
  /**
   * 脚本开始
   * @type {[type]}
   */
  grunt.initConfig({
    pkg: grunt.file.readJSON('package.json'),
    /**
     * 构建在开发文件中的目录
     */
    cssmin: {
      /**
       * 合并并压缩
       */
      minify: {
        src: cssArr.concat(['css/*.css']),
        dest: dest.project + '/' + name + '/css/' + name + '.min.css'
      }
    },
    /**
     * 如果使用less，
     * @type {Object}
     */
    less: {
      compile: {
        files: {
          './css/less.css': './css/*.less'
        }
      }
    },
    /**
     * css语法检测,默认不执行，自行添加任务
     */
    csslint: {
      all: ['./css/*.css']
    },
    /**
     * js语法检测,默认不执行，自行添加任务
     */
    jshint: {
      all: ['./js/*.js']
    },
    /**
     * copy任务：按照构建图的规则把本地css下图片，js，images文件夹里的内容复制到即将对接到构建目录的project文件夹里，temp和*.html文件复制到即将对接到构建目录里的demo目录下。
     */
    copy: {
      cssimages: {
        expand: true,
        cwd: './css/images',
        src: ['*'],
        dest: dest.project + '/' + name + '/css/images'
      },
      js: {
        expand: true,
        src: jsArr.concat(['./js/*.js']),
        dest: dest.project + '/' + name + '/js',
        flatten: true,
        filter: 'isFile'
      },
      images: {
        expand: true,
        cwd: './images/',
        src: '**',
        dest: dest.project + '/' + name + '/images/'
      },
      htmls: {
        expand: true,
        src: ['./*.html', './temp/**', '!./_*.html'],
        dest: dest.demo + '/' + name,
        options: {
          process: function(content, filepath) {
            var $ = cheerio.load(content);

            /**
             * css合并构建方式。移除所有css链接，并加上最终合并之后的链接
             */

            $('link')
              .filter(function() {
                if (this.attr('href') != undefined) {
                  return !this.attr('href').match(/^http|https/) && (this.attr('rel') == 'stylesheet') && this.attr('href') ? /.css$|.css\?_component$/.test(this.attr('href')) : false;
                };
              })
              .replaceWith('<link rel="stylesheet" href="/project/' + name + '/css/' + name + '.min.css">')

            /**
             * js非合并，但资源定位
             */
            $('script')
              .filter(function() {
                if (this.attr('src') != undefined) {
                  return !this.attr('src').match(/^http|https/) && (this.attr('type') == 'text/javascript' || !this.attr('type')) && this.attr('src') ? /.js|.js\?_component$/.test(this.attr('src')) : false;
                };
              })
              .attr('src', function(k, v) {
                var reg = /.*components(?=\/).*(\/js\/.*)/
                if (reg.test(v)) {
                  return '/project/' + name + v.replace(reg, '$1');
                } else {
                  return '/project/' + name + '/' + this.attr('src')
                }
              })
            /**
             * 如果使用requirejs 替换配置文件
            .attr('data-main', function(k,v) {
              if (v!=undefined) {
                return '/project/' + name + '/'+this.attr('data-main')
              }else{
                return ''
              }
            })
            */
            /**
             * 定位前景非零时图片
             */
            $('img[src^="images"]')
              .filter(function() {
                if (this.attr('src') != undefined) {
                  return !this.attr('src').match(/^http|https/)
                };
              })
              .attr('src', function() {
                return '/project/' + name + '/' + this.attr('src')
              })

            /**
             * html 模块加载，匹配以_开头的html文件。并合到所引用的dom中。
             */
            var reg = /<\!--#include\s*virtual="(_.*)"-->/g;
            var matched = $.html().match(reg);
            var dom = $.html();
            if (Object.prototype.toString.call(matched) != '[object Null]') {
              matched.forEach(function(val) {
                var filename = val.replace(reg, '$1');
                dom = dom.replace(val, fs.readFileSync(filename));
              })
              return dom;
            }
          }
        }
      }
    },
    clean: {
      dirs: {
        src: [dest.demo, dest.project],
        options: {
          force: true
        }
      }
    },
    watch: {
      css: {
        files: '../help/**',
        tasks: ['cssmin', 'copy']
      }
    }
  });
  /**
   * @jshint，js语法检测任务，默认不执行，如果要执行在Task里面加上任务
   * @cssmin, css压缩合并任务，默认执行
   */
  grunt.loadNpmTasks('grunt-contrib-csslint');
  grunt.loadNpmTasks('grunt-contrib-jshint');
  grunt.loadNpmTasks('grunt-contrib-cssmin');
  grunt.loadNpmTasks('grunt-contrib-less');
  grunt.loadNpmTasks('grunt-contrib-copy');
  grunt.loadNpmTasks('grunt-contrib-clean');
  grunt.loadNpmTasks('grunt-contrib-watch');
  grunt.registerTask('default', ['clean', 'cssmin', 'copy', 'watch']);
}
